@@ -259,7 +259,7 @@ impl Spec {
                 .set_case_insensitive(shell.options.case_insensitive_pathname_expansion);
 
             let expansions = pattern.expand(
-                shell.working_dir.as_path(),
+                shell.working_dir(),
                 Some(&patterns::Pattern::accept_all_expand_filter),
                 &patterns::FilenameExpansionOptions::default(),
             )?;
@@ -410,7 +410,7 @@ impl Spec {
                     tracing::debug!(target: trace_categories::COMPLETION, "unimplemented: complete -A binding");
                 }
                 CompleteAction::Builtin => {
-                    for name in shell.builtins.keys() {
+                    for name in shell.builtins().keys() {
                         if name.starts_with(token) {
                             candidates.insert(name.to_owned());
                         }
@@ -426,14 +426,14 @@ impl Spec {
                     candidates.append(&mut file_completions);
                 }
                 CompleteAction::Disabled => {
-                    for (name, registration) in &shell.builtins {
+                    for (name, registration) in shell.builtins() {
                         if registration.disabled && name.starts_with(token) {
                             candidates.insert(name.to_owned());
                         }
                     }
                 }
                 CompleteAction::Enabled => {
-                    for (name, registration) in &shell.builtins {
+                    for (name, registration) in shell.builtins() {
                         if !registration.disabled && name.starts_with(token) {
                             candidates.insert(name.to_owned());
                         }
@@ -452,7 +452,7 @@ impl Spec {
                     candidates.append(&mut file_completions);
                 }
                 CompleteAction::Function => {
-                    for (name, _) in shell.funcs.iter() {
+                    for (name, _) in shell.funcs().iter() {
                         candidates.insert(name.to_owned());
                     }
                 }
@@ -465,7 +465,7 @@ impl Spec {
                 }
                 CompleteAction::HelpTopic => {
                     // For now, we only have help topics for built-in commands.
-                    for name in shell.builtins.keys() {
+                    for name in shell.builtins().keys() {
                         if name.starts_with(token) {
                             candidates.insert(name.to_owned());
                         }
@@ -482,7 +482,7 @@ impl Spec {
                 }
                 CompleteAction::Job => {
                     for job in &shell.jobs.jobs {
-                        let command_name = job.get_command_name();
+                        let command_name = job.command_name();
                         if command_name.starts_with(token) {
                             candidates.insert(command_name.to_owned());
                         }
@@ -498,7 +498,7 @@ impl Spec {
                 CompleteAction::Running => {
                     for job in &shell.jobs.jobs {
                         if matches!(job.state, jobs::JobState::Running) {
-                            let command_name = job.get_command_name();
+                            let command_name = job.command_name();
                             if command_name.starts_with(token) {
                                 candidates.insert(command_name.to_owned());
                             }
@@ -509,16 +509,18 @@ impl Spec {
                     tracing::debug!(target: trace_categories::COMPLETION, "unimplemented: complete -A service");
                 }
                 CompleteAction::SetOpt => {
-                    for (name, _) in namedoptions::SET_O_OPTIONS.iter() {
-                        if name.starts_with(token) {
-                            candidates.insert((*name).to_owned());
+                    for option in namedoptions::options(namedoptions::ShellOptionKind::SetO).iter()
+                    {
+                        if option.name.starts_with(token) {
+                            candidates.insert(option.name.to_owned());
                         }
                     }
                 }
                 CompleteAction::ShOpt => {
-                    for (name, _) in namedoptions::SHOPT_OPTIONS.iter() {
-                        if name.starts_with(token) {
-                            candidates.insert((*name).to_owned());
+                    for option in namedoptions::options(namedoptions::ShellOptionKind::Shopt).iter()
+                    {
+                        if option.name.starts_with(token) {
+                            candidates.insert(option.name.to_owned());
                         }
                     }
                 }
@@ -532,9 +534,9 @@ impl Spec {
                 CompleteAction::Stopped => {
                     for job in &shell.jobs.jobs {
                         if matches!(job.state, jobs::JobState::Stopped) {
-                            let command_name = job.get_command_name();
+                            let command_name = job.command_name();
                             if command_name.starts_with(token) {
-                                candidates.insert(job.get_command_name().to_owned());
+                                candidates.insert(job.command_name().to_owned());
                             }
                         }
                     }
@@ -892,7 +894,6 @@ impl Config {
     /// * `shell` - The shell instance to use for completion generation.
     /// * `input` - The input line for which completions are being generated.
     /// * `position` - The 0-based index of the cursor in the input line.
-    #[expect(clippy::cast_sign_loss)]
     #[expect(clippy::string_slice)]
     pub async fn get_completions(
         &self,
@@ -905,7 +906,7 @@ impl Config {
         // Make a best-effort attempt to tokenize.
         let tokens = Self::tokenize_input_for_completion(shell, input);
 
-        let cursor = i32::try_from(position)?;
+        let cursor = position;
         let mut preceding_token = None;
         let mut completion_prefix = "";
         let mut insertion_index = cursor;
@@ -937,7 +938,7 @@ impl Config {
                 insertion_index = token.location().start.index;
 
                 // Update prefix.
-                let offset_into_token = (cursor - insertion_index) as usize;
+                let offset_into_token = cursor - insertion_index;
                 let token_str = token.to_str();
                 completion_prefix = &token_str[..offset_into_token];
 
@@ -965,7 +966,7 @@ impl Config {
         let mut restart_count = 0;
         while matches!(result, Answer::RestartCompletionProcess) {
             if restart_count > MAX_RESTARTS {
-                tracing::error!("possible infinite loop detected in completion process");
+                tracing::warn!("possible infinite loop detected in completion process");
                 break;
             }
 
@@ -988,13 +989,13 @@ impl Config {
 
         match result {
             Answer::Candidates(candidates, options) => Ok(Completions {
-                insertion_index: insertion_index as usize,
+                insertion_index,
                 delete_count: completion_prefix.len(),
                 candidates,
                 options,
             }),
             Answer::RestartCompletionProcess => Ok(Completions {
-                insertion_index: insertion_index as usize,
+                insertion_index,
                 delete_count: 0,
                 candidates: IndexSet::new(),
                 options: ProcessingOptions::default(),
@@ -1006,7 +1007,7 @@ impl Config {
         const FALLBACK: &str = " \t\n\"\'@><=;|&(:";
 
         let delimiter_str = shell
-            .get_env_str("COMP_WORDBREAKS")
+            .env_str("COMP_WORDBREAKS")
             .unwrap_or_else(|| FALLBACK.into());
 
         let delimiters: Vec<_> = delimiter_str.chars().collect();
@@ -1078,7 +1079,7 @@ async fn get_file_completions(
 
     let glob = std::format!("{expanded_token}*");
 
-    let path_filter = |path: &Path| !must_be_dir || shell.get_absolute_path(path).is_dir();
+    let path_filter = |path: &Path| !must_be_dir || shell.absolute_path(path).is_dir();
 
     let pattern = patterns::Pattern::from(glob)
         .set_extended_globbing(shell.options.extended_globbing)
@@ -1086,7 +1087,7 @@ async fn get_file_completions(
 
     pattern
         .expand(
-            shell.working_dir.as_path(),
+            shell.working_dir(),
             Some(&path_filter),
             &patterns::FilenameExpansionOptions::default(),
         )
@@ -1129,14 +1130,14 @@ async fn get_completions_using_basic_lookup(shell: &Shell, context: &Context<'_>
         candidates.append(&mut command_completions);
 
         // Add built-in commands.
-        for (name, registration) in &shell.builtins {
+        for (name, registration) in shell.builtins() {
             if !registration.disabled && name.starts_with(context.token_to_complete) {
                 candidates.insert(name.to_owned());
             }
         }
 
         // Add shell functions.
-        for (name, _) in shell.funcs.iter() {
+        for (name, _) in shell.funcs().iter() {
             if name.starts_with(context.token_to_complete) {
                 candidates.insert(name.to_owned());
             }
@@ -1171,21 +1172,19 @@ async fn get_completions_using_basic_lookup(shell: &Shell, context: &Context<'_>
     Answer::Candidates(candidates, ProcessingOptions::default())
 }
 
-#[expect(clippy::cast_possible_truncation)]
-#[expect(clippy::cast_possible_wrap)]
 fn simple_tokenize_by_delimiters(input: &str, delimiters: &[char]) -> Vec<brush_parser::Token> {
     //
     // This is an overly naive tokenization.
     //
 
     let mut tokens = vec![];
-    let mut start: i32 = 0;
+    let mut start = 0;
 
     for piece in input.split_inclusive(delimiters) {
-        let next_start = start + piece.len() as i32;
+        let next_start = start + piece.len();
 
         let piece = piece.strip_suffix(delimiters).unwrap_or(piece);
-        let end: i32 = start + piece.len() as i32;
+        let end = start + piece.len();
         tokens.push(brush_parser::Token::Word(
             piece.to_string(),
             brush_parser::TokenLocation {
@@ -1193,12 +1192,14 @@ fn simple_tokenize_by_delimiters(input: &str, delimiters: &[char]) -> Vec<brush_
                     index: start,
                     line: 1,
                     column: start + 1,
-                },
+                }
+                .into(),
                 end: brush_parser::SourcePosition {
                     index: end,
                     line: 1,
                     column: end + 1,
-                },
+                }
+                .into(),
             },
         ));
 
