@@ -2,7 +2,8 @@ use brush_parser::ast;
 use std::path::Path;
 
 use crate::{
-    ExecutionParameters, Shell, arithmetic, env, error, escape, expansion, namedoptions, patterns,
+    ExecutionParameters, Shell, ShellFd, arithmetic, env, error, escape, expansion, namedoptions,
+    patterns,
     sys::{
         fs::{MetadataExt, PathExt},
         users,
@@ -53,10 +54,13 @@ async fn apply_unary_predicate(
 
     if shell.options.print_commands_and_arguments {
         shell
-            .trace_command(std::format!(
-                "[[ {op} {} ]]",
-                escape::quote_if_needed(&expanded_operand, escape::QuoteMode::SingleQuote)
-            ))
+            .trace_command(
+                params,
+                std::format!(
+                    "[[ {op} {} ]]",
+                    escape::quote_if_needed(&expanded_operand, escape::QuoteMode::SingleQuote)
+                ),
+            )
             .await?;
     }
 
@@ -74,47 +78,47 @@ pub(crate) fn apply_unary_predicate_to_str(
         ast::UnaryPredicate::StringHasNonZeroLength => Ok(!operand.is_empty()),
         ast::UnaryPredicate::StringHasZeroLength => Ok(operand.is_empty()),
         ast::UnaryPredicate::FileExists => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists())
         }
         ast::UnaryPredicate::FileExistsAndIsBlockSpecialFile => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_block_device())
         }
         ast::UnaryPredicate::FileExistsAndIsCharSpecialFile => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_char_device())
         }
         ast::UnaryPredicate::FileExistsAndIsDir => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.is_dir())
         }
         ast::UnaryPredicate::FileExistsAndIsRegularFile => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.is_file())
         }
         ast::UnaryPredicate::FileExistsAndIsSetgid => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_setgid())
         }
         ast::UnaryPredicate::FileExistsAndIsSymlink => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.is_symlink())
         }
         ast::UnaryPredicate::FileExistsAndHasStickyBit => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_sticky_bit())
         }
         ast::UnaryPredicate::FileExistsAndIsFifo => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_fifo())
         }
         ast::UnaryPredicate::FileExistsAndIsReadable => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.readable())
         }
         ast::UnaryPredicate::FileExistsAndIsNotZeroLength => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             if let Ok(metadata) = path.metadata() {
                 Ok(metadata.len() > 0)
             } else {
@@ -122,8 +126,8 @@ pub(crate) fn apply_unary_predicate_to_str(
             }
         }
         ast::UnaryPredicate::FdIsOpenTerminal => {
-            if let Ok(fd) = operand.parse::<u32>() {
-                if let Some(open_file) = params.open_files.get(fd) {
+            if let Ok(fd) = operand.parse::<ShellFd>() {
+                if let Some(open_file) = params.try_fd(shell, fd) {
                     Ok(open_file.is_term())
                 } else {
                     Ok(false)
@@ -133,19 +137,19 @@ pub(crate) fn apply_unary_predicate_to_str(
             }
         }
         ast::UnaryPredicate::FileExistsAndIsSetuid => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_setuid())
         }
         ast::UnaryPredicate::FileExistsAndIsWritable => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.writable())
         }
         ast::UnaryPredicate::FileExistsAndIsExecutable => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.executable())
         }
         ast::UnaryPredicate::FileExistsAndOwnedByEffectiveGroupId => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             if !path.exists() {
                 return Ok(false);
             }
@@ -157,7 +161,7 @@ pub(crate) fn apply_unary_predicate_to_str(
             error::unimp("unary extended test predicate: FileExistsAndModifiedSinceLastRead")
         }
         ast::UnaryPredicate::FileExistsAndOwnedByEffectiveUserId => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             if !path.exists() {
                 return Ok(false);
             }
@@ -166,13 +170,15 @@ pub(crate) fn apply_unary_predicate_to_str(
             Ok(md.uid() == users::get_effective_uid()?)
         }
         ast::UnaryPredicate::FileExistsAndIsSocket => {
-            let path = shell.get_absolute_path(Path::new(operand));
+            let path = shell.absolute_path(Path::new(operand));
             Ok(path.exists_and_is_socket())
         }
         ast::UnaryPredicate::ShellOptionEnabled => {
             let shopt_name = operand;
-            if let Some(option) = namedoptions::SET_O_OPTIONS.get(shopt_name) {
-                Ok((option.getter)(&shell.options))
+            if let Some(option) =
+                namedoptions::options(namedoptions::ShellOptionKind::SetO).get(shopt_name)
+            {
+                Ok(option.get(&shell.options))
             } else {
                 Ok(false)
             }
@@ -202,7 +208,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {s} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {s} {op} {right} ]]"))
                     .await?;
             }
 
@@ -241,7 +247,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -253,7 +259,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -265,7 +271,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {s} {op} {substring} ]]"))
+                    .trace_command(params, std::format!("[[ {s} {op} {substring} ]]"))
                     .await?;
             }
 
@@ -277,7 +283,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -289,7 +295,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -301,7 +307,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -313,7 +319,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -326,7 +332,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -341,7 +347,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -355,7 +361,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -369,7 +375,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -383,7 +389,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -397,7 +403,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -411,7 +417,7 @@ async fn apply_binary_predicate(
 
             if shell.options.print_commands_and_arguments {
                 shell
-                    .trace_command(std::format!("[[ {left} {op} {right} ]]"))
+                    .trace_command(params, std::format!("[[ {left} {op} {right} ]]"))
                     .await?;
             }
 
@@ -435,7 +441,7 @@ async fn apply_binary_predicate(
                     escape::QuoteMode::BackslashEscape,
                 );
                 shell
-                    .trace_command(std::format!("[[ {s} {op} {escaped_right} ]]"))
+                    .trace_command(params, std::format!("[[ {s} {op} {escaped_right} ]]"))
                     .await?;
             }
 
@@ -455,7 +461,7 @@ async fn apply_binary_predicate(
                     escape::QuoteMode::BackslashEscape,
                 );
                 shell
-                    .trace_command(std::format!("[[ {s} {op} {escaped_right} ]]"))
+                    .trace_command(params, std::format!("[[ {s} {op} {escaped_right} ]]"))
                     .await?;
             }
 
@@ -557,8 +563,8 @@ fn left_file_is_older_or_does_not_exist_when_right_does(
     right: impl AsRef<str>,
 ) -> Result<bool, error::Error> {
     let (l_path, r_path) = (
-        shell.get_absolute_path(Path::new(left.as_ref())),
-        shell.get_absolute_path(Path::new(right.as_ref())),
+        shell.absolute_path(Path::new(left.as_ref())),
+        shell.absolute_path(Path::new(right.as_ref())),
     );
 
     match (l_path.metadata(), r_path.metadata()) {
@@ -574,8 +580,8 @@ fn left_file_is_newer_or_exists_when_right_does_not(
     right: impl AsRef<str>,
 ) -> Result<bool, error::Error> {
     let (l_path, r_path) = (
-        shell.get_absolute_path(Path::new(left.as_ref())),
-        shell.get_absolute_path(Path::new(right.as_ref())),
+        shell.absolute_path(Path::new(left.as_ref())),
+        shell.absolute_path(Path::new(right.as_ref())),
     );
 
     match (l_path.metadata(), r_path.metadata()) {
@@ -591,8 +597,8 @@ fn files_refer_to_same_device_and_inode_numbers(
     right: impl AsRef<str>,
 ) -> Result<bool, error::Error> {
     let (l_path, r_path) = (
-        shell.get_absolute_path(Path::new(left.as_ref())),
-        shell.get_absolute_path(Path::new(right.as_ref())),
+        shell.absolute_path(Path::new(left.as_ref())),
+        shell.absolute_path(Path::new(right.as_ref())),
     );
 
     if !l_path.readable() || !r_path.readable() {
