@@ -6,12 +6,14 @@
 #![allow(clippy::unused_async)]
 #![allow(clippy::unused_self)]
 
-pub(crate) mod fs;
-pub(crate) mod input;
+pub mod commands;
+pub mod fd;
+pub mod fs;
+pub mod input;
 pub(crate) mod network;
 pub(crate) mod pipes;
-pub(crate) mod process;
-pub(crate) mod resource;
-pub(crate) mod signal;
-pub(crate) mod terminal;
+pub mod process;
+pub mod resource;
+pub mod signal;
+pub mod terminal;
 pub(crate) mod users;
