@@ -1,4 +1,7 @@
-#[cfg(not(unix))]
+//! Filesystem utilities.
+
+use crate::error;
+
 impl crate::sys::fs::PathExt for std::path::Path {
     fn readable(&self) -> bool {
         true
@@ -45,7 +48,7 @@ impl crate::sys::fs::PathExt for std::path::Path {
     }
 }
 
-pub(crate) trait StubMetadataExt {
+pub(crate) trait MetadataExt {
     fn gid(&self) -> u32 {
         0
     }
@@ -55,12 +58,21 @@ pub(crate) trait StubMetadataExt {
     }
 }
 
-impl StubMetadataExt for std::fs::Metadata {}
+impl MetadataExt for std::fs::Metadata {}
 
 pub(crate) fn get_default_executable_search_paths() -> Vec<String> {
     vec![]
 }
 
-pub(crate) fn get_default_standard_utils_paths() -> Vec<String> {
+/// Returns the default paths where standard Unix utilities are typically installed.
+/// This is a stub implementation that returns an empty vector.
+pub fn get_default_standard_utils_paths() -> Vec<String> {
     vec![]
 }
+
+/// Opens a null file that will discard all I/O.
+///
+/// This is a stub implementation that returns an error.
+pub fn open_null_file() -> Result<std::fs::File, error::Error> {
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("opening null file").into())
+}
