@@ -13,16 +13,24 @@ pub(crate) fn is_root() -> bool {
     false
 }
 
+pub(crate) fn get_current_uid() -> Result<u32, error::Error> {
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("getting current uid").into())
+}
+
+pub(crate) fn get_current_gid() -> Result<u32, error::Error> {
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("getting current gid").into())
+}
+
 pub(crate) fn get_effective_uid() -> Result<u32, error::Error> {
-    error::unimp("get effective uid")
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("getting effective uid").into())
 }
 
 pub(crate) fn get_effective_gid() -> Result<u32, error::Error> {
-    error::unimp("get effective gid")
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("getting effective gid").into())
 }
 
 pub(crate) fn get_current_username() -> Result<String, error::Error> {
-    error::unimp("get current username")
+    Err(error::ErrorKind::NotSupportedOnThisPlatform("getting current username").into())
 }
 
 pub(crate) fn get_user_group_ids() -> Result<Vec<u32>, error::Error> {
