@@ -1,6 +1,17 @@
+//! Shell test conditional expressions
+
 use crate::{ExecutionParameters, Shell, error, extendedtests};
 
-pub(crate) fn eval_test_expr(
+/// Evaluate the given test expression within the provided shell and
+/// execution context. Returns true if the expression evaluates to true,
+/// false otherwise.
+///
+/// # Arguments
+///
+/// * `expr` - The test expression to evaluate.
+/// * `shell` - The shell context in which to evaluate the expression.
+/// * `params` - The execution parameters to use during evaluation.
+pub fn eval_expr(
     expr: &brush_parser::ast::TestExpr,
     shell: &mut Shell,
     params: &ExecutionParameters,
@@ -9,13 +20,13 @@ pub(crate) fn eval_test_expr(
         brush_parser::ast::TestExpr::False => Ok(false),
         brush_parser::ast::TestExpr::Literal(s) => Ok(!s.is_empty()),
         brush_parser::ast::TestExpr::And(left, right) => {
-            Ok(eval_test_expr(left, shell, params)? && eval_test_expr(right, shell, params)?)
+            Ok(eval_expr(left, shell, params)? && eval_expr(right, shell, params)?)
         }
         brush_parser::ast::TestExpr::Or(left, right) => {
-            Ok(eval_test_expr(left, shell, params)? || eval_test_expr(right, shell, params)?)
+            Ok(eval_expr(left, shell, params)? || eval_expr(right, shell, params)?)
         }
-        brush_parser::ast::TestExpr::Not(expr) => Ok(!eval_test_expr(expr, shell, params)?),
-        brush_parser::ast::TestExpr::Parenthesized(expr) => eval_test_expr(expr, shell, params),
+        brush_parser::ast::TestExpr::Not(expr) => Ok(!eval_expr(expr, shell, params)?),
+        brush_parser::ast::TestExpr::Parenthesized(expr) => eval_expr(expr, shell, params),
         brush_parser::ast::TestExpr::UnaryTest(op, operand) => {
             extendedtests::apply_unary_predicate_to_str(op, operand, shell, params)
         }
