@@ -1,11 +1,11 @@
-use std::io::{IsTerminal, Write};
+use std::io::IsTerminal;
 
 use crate::{
     ShellError, completion,
     interactive_shell::{InteractivePrompt, InteractiveShell, ReadResult},
 };
 
-use super::term_line_reader;
+use super::{non_term_line_reader, term_line_reader};
 
 /// Represents a basic shell capable of interactive usage, with primitive support
 /// for completion and test-focused automation via pexpect and similar technologies.
@@ -19,8 +19,8 @@ impl BasicShell {
     /// # Arguments
     ///
     /// * `options` - Options for creating the interactive shell.
-    pub async fn new(options: &crate::Options) -> Result<Self, ShellError> {
-        let shell = brush_core::Shell::new(&options.shell).await?;
+    pub async fn new(options: crate::Options) -> Result<Self, ShellError> {
+        let shell = brush_core::Shell::new(options.shell).await?;
         Ok(Self { shell })
     }
 }
@@ -37,17 +37,33 @@ impl InteractiveShell for BasicShell {
     }
 
     fn read_line(&mut self, prompt: InteractivePrompt) -> Result<ReadResult, ShellError> {
-        self.display_prompt(&prompt)?;
+        if std::io::stdin().is_terminal() {
+            self.read_line_via(&term_line_reader::TermLineReader::new()?, &prompt)
+        } else {
+            self.read_line_via(&non_term_line_reader::NonTermLineReader, &prompt)
+        }
+    }
+}
 
+impl BasicShell {
+    fn read_line_via<R: super::LineReader>(
+        &mut self,
+        reader: &R,
+        prompt: &InteractivePrompt,
+    ) -> Result<ReadResult, ShellError> {
+        let mut prompt_to_use = self.should_display_prompt().then_some(&prompt);
         let mut result = String::new();
 
         loop {
-            match self.read_input_line(&prompt)? {
+            match reader.read_line(prompt_to_use.map(|p| p.prompt.as_str()), |line, cursor| {
+                self.generate_completions(line, cursor)
+            })? {
                 ReadResult::Input(s) => {
                     result.push_str(s.as_str());
                     if self.is_valid_input(result.as_str()) {
                         break;
                     }
+                    prompt_to_use = None;
                 }
                 ReadResult::BoundCommand(s) => {
                     result.push_str(s.as_str());
@@ -65,42 +81,12 @@ impl InteractiveShell for BasicShell {
 
         Ok(ReadResult::Input(result))
     }
-}
 
-impl BasicShell {
     #[expect(clippy::unused_self)]
     fn should_display_prompt(&self) -> bool {
         std::io::stdin().is_terminal()
     }
 
-    fn display_prompt(&self, prompt: &InteractivePrompt) -> Result<(), ShellError> {
-        if self.should_display_prompt() {
-            eprint!("{}", prompt.prompt);
-            std::io::stderr().flush()?;
-        }
-
-        Ok(())
-    }
-
-    fn read_input_line(&mut self, prompt: &InteractivePrompt) -> Result<ReadResult, ShellError> {
-        if std::io::stdin().is_terminal() {
-            term_line_reader::read_line(prompt.prompt.as_str(), |line, cursor| {
-                self.generate_completions(line, cursor)
-            })
-        } else {
-            let mut input = String::new();
-            let bytes_read = std::io::stdin()
-                .read_line(&mut input)
-                .map_err(|_err| ShellError::InputError)?;
-
-            if bytes_read == 0 {
-                Ok(ReadResult::Eof)
-            } else {
-                Ok(ReadResult::Input(input))
-            }
-        }
-    }
-
     fn is_valid_input(&self, input: &str) -> bool {
         match self.shell.parse_string(input.to_owned()) {
             Err(brush_parser::ParseError::Tokenizing { inner, position: _ })
