@@ -72,7 +72,7 @@ pub trait InteractiveShell: Send {
         async {
             // Acquire terminal control if stdin is a terminal.
             if std::io::stdin().is_terminal() {
-                brush_core::TerminalControl::acquire()?;
+                brush_core::terminal::TerminalControl::acquire()?;
             }
 
             let mut announce_exit = self.shell().as_ref().options.interactive;
@@ -81,21 +81,24 @@ pub trait InteractiveShell: Send {
                 let result = self.run_interactively_once().await?;
                 match result {
                     InteractiveExecutionResult::Executed(brush_core::ExecutionResult {
-                        exit_shell,
-                        return_from_function_or_script,
+                        next_control_flow: brush_core::results::ExecutionControlFlow::ExitShell,
                         ..
                     }) => {
-                        if exit_shell {
-                            break;
-                        }
-
-                        if return_from_function_or_script {
-                            tracing::error!("return from non-function/script");
-                        }
+                        break;
                     }
-                    InteractiveExecutionResult::Failed(e) => {
+                    InteractiveExecutionResult::Executed(brush_core::ExecutionResult {
+                        next_control_flow:
+                            brush_core::results::ExecutionControlFlow::ReturnFromFunctionOrScript,
+                        ..
+                    }) => {
+                        tracing::error!("return from non-function/script");
+                    }
+                    InteractiveExecutionResult::Executed(_) => {}
+                    InteractiveExecutionResult::Failed(err) => {
                         // Report the error, but continue to execute.
-                        tracing::error!("error: {:#}", e);
+                        let shell = self.shell();
+                        let mut stderr = shell.as_ref().stderr();
+                        let _ = shell.as_ref().display_error(&mut stderr, &err).await;
                     }
                     InteractiveExecutionResult::Eof => {
                         break;
@@ -118,6 +121,9 @@ pub trait InteractiveShell: Send {
                 tracing::debug!("couldn't save history: {e}");
             }
 
+            // Give the shell an opportunity to perform any on-exit operations.
+            self.shell_mut().as_mut().on_exit().await?;
+
             Ok(())
         }
     }
@@ -156,10 +162,10 @@ pub trait InteractiveShell: Send {
                 ReadResult::Eof => Ok(InteractiveExecutionResult::Eof),
                 ReadResult::Interrupted => {
                     let mut shell_mut = self.shell_mut();
-                    shell_mut.as_mut().last_exit_status = 130;
-                    Ok(InteractiveExecutionResult::Executed(
-                        brush_core::ExecutionResult::new(130),
-                    ))
+                    let result: brush_core::ExecutionResult =
+                        brush_core::ExecutionExitCode::Interrupted.into();
+                    *shell_mut.as_mut().last_exit_status_mut() = result.exit_code.into();
+                    Ok(InteractiveExecutionResult::Executed(result))
                 }
             }
         }
@@ -173,10 +179,14 @@ pub trait InteractiveShell: Send {
     ) -> impl std::future::Future<Output = Result<InteractiveExecutionResult, ShellError>> + Send
     {
         async move {
+            // See if the the user interface has a non-empty read buffer.
             let buffer_info = self.get_read_buffer();
 
             let mut shell_mut = self.shell_mut();
 
+            // If the user interface did, in fact, have a non-empty read buffer,
+            // then reflect it to the shell in case any shell code wants to
+            // process and/or transform the buffer.
             let nonempty_buffer = if let Some((buffer, cursor)) = buffer_info {
                 if !buffer.is_empty() {
                     shell_mut.as_mut().set_edit_buffer(buffer, cursor)?;
@@ -210,12 +220,17 @@ pub trait InteractiveShell: Send {
                 Err(e) => Ok(InteractiveExecutionResult::Failed(e)),
             };
 
-            if nonempty_buffer {
-                let (updated_buffer, updated_cursor) = shell_mut
-                    .as_mut()
-                    .pop_edit_buffer()?
-                    .unwrap_or((String::new(), 0));
+            // See if the shell has input buffer state that we need to reflect back to
+            // the user interface. It may be state that originally came from the user
+            // interface, or it may be state that was programmatically generated by
+            // the command we just executed.
+            let mut buffer_and_cursor = shell_mut.as_mut().pop_edit_buffer()?;
 
+            if buffer_and_cursor.is_none() && nonempty_buffer {
+                buffer_and_cursor = Some((String::new(), 0));
+            }
+
+            if let Some((updated_buffer, updated_cursor)) = buffer_and_cursor {
                 drop(shell_mut);
 
                 self.set_read_buffer(updated_buffer, updated_cursor);
@@ -228,7 +243,7 @@ pub trait InteractiveShell: Send {
 
 async fn run_pre_prompt_commands(shell: &mut brush_core::Shell) -> Result<(), ShellError> {
     // If there's a variable called PROMPT_COMMAND, then run it first.
-    if let Some(prompt_cmd_var) = shell.get_env_var("PROMPT_COMMAND") {
+    if let Some(prompt_cmd_var) = shell.env_var("PROMPT_COMMAND") {
         match prompt_cmd_var.value() {
             brush_core::ShellValue::String(cmd_str) => {
                 run_pre_prompt_command(shell, cmd_str.to_owned()).await?;
@@ -252,7 +267,7 @@ async fn run_pre_prompt_command(
     prompt_cmd: impl Into<String>,
 ) -> Result<(), ShellError> {
     // Save (and later restore) the last exit status.
-    let prev_last_result = shell.last_exit_status;
+    let prev_last_result = shell.last_result();
     let prev_last_pipeline_statuses = shell.last_pipeline_statuses.clone();
 
     // Run the command.
@@ -261,7 +276,7 @@ async fn run_pre_prompt_command(
 
     // Restore the last exit status.
     shell.last_pipeline_statuses = prev_last_pipeline_statuses;
-    shell.last_exit_status = prev_last_result;
+    *shell.last_exit_status_mut() = prev_last_result;
 
     Ok(())
 }
