@@ -3,10 +3,27 @@
 
 use std::fmt::{Display, Write};
 
-use crate::tokenizer;
+use crate::{TokenLocation, tokenizer};
 
 const DISPLAY_INDENT: &str = "    ";
 
+/// Provides the source location for the syntax item
+pub trait SourceLocation {
+    /// The location of the syntax item, when known
+    fn location(&self) -> Option<TokenLocation>;
+}
+
+pub(crate) fn maybe_location(
+    start: Option<&TokenLocation>,
+    end: Option<&TokenLocation>,
+) -> Option<TokenLocation> {
+    if let (Some(s), Some(e)) = (start, end) {
+        Some(TokenLocation::within(s, e))
+    } else {
+        None
+    }
+}
+
 /// Represents a complete shell program.
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
@@ -17,6 +34,20 @@ pub struct Program {
     pub complete_commands: Vec<CompleteCommand>,
 }
 
+impl SourceLocation for Program {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self
+            .complete_commands
+            .first()
+            .and_then(SourceLocation::location);
+        let end = self
+            .complete_commands
+            .last()
+            .and_then(SourceLocation::location);
+        maybe_location(start.as_ref(), end.as_ref())
+    }
+}
+
 impl Display for Program {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for complete_command in &self.complete_commands {
@@ -32,6 +63,7 @@ pub type CompleteCommand = CompoundList;
 /// Represents a complete shell command item.
 pub type CompleteCommandItem = CompoundListItem;
 
+// TODO: decide if we want to trace this location or consider it a whitespace separator
 /// Indicates whether the preceding command is executed synchronously or asynchronously.
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
@@ -65,6 +97,19 @@ pub struct AndOrList {
     pub additional: Vec<AndOr>,
 }
 
+impl SourceLocation for AndOrList {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.first.location();
+        let last = self.additional.last();
+        let end = last.and_then(SourceLocation::location);
+
+        match (start, end) {
+            (Some(s), Some(e)) => Some(TokenLocation::within(&s, &e)),
+            (start, _) => start,
+        }
+    }
+}
+
 impl Display for AndOrList {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.first)?;
@@ -168,6 +213,16 @@ pub enum AndOr {
     Or(Pipeline),
 }
 
+// TODO: add a loc
+impl SourceLocation for AndOr {
+    fn location(&self) -> Option<TokenLocation> {
+        match self {
+            Self::And(p) => p.location(),
+            Self::Or(p) => p.location(),
+        }
+    }
+}
+
 impl Display for AndOr {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -183,9 +238,25 @@ impl Display for AndOr {
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub enum PipelineTimed {
     /// The pipeline should be timed with bash-like output.
-    Timed,
+    Timed(TokenLocation),
     /// The pipeline should be timed with POSIX-like output.
-    TimedWithPosixOutput,
+    TimedWithPosixOutput(TokenLocation),
+}
+
+impl SourceLocation for PipelineTimed {
+    fn location(&self) -> Option<TokenLocation> {
+        match self {
+            Self::Timed(t) => Some(t.to_owned()),
+            Self::TimedWithPosixOutput(t) => Some(t.to_owned()),
+        }
+    }
+}
+
+impl PipelineTimed {
+    /// Returns true if the pipeline should be timed with POSIX-like output.
+    pub const fn is_posix_output(&self) -> bool {
+        matches!(self, Self::TimedWithPosixOutput(_))
+    }
 }
 
 /// A pipeline of commands, where each command's output is passed as standard input
@@ -206,6 +277,19 @@ pub struct Pipeline {
     pub seq: Vec<Command>,
 }
 
+impl SourceLocation for Pipeline {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self
+            .timed
+            .as_ref()
+            .and_then(SourceLocation::location)
+            .or_else(|| self.seq.first().and_then(SourceLocation::location));
+        let end = self.seq.last().and_then(SourceLocation::location);
+
+        maybe_location(start.as_ref(), end.as_ref())
+    }
+}
+
 impl Display for Pipeline {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         if self.bang {
@@ -235,7 +319,23 @@ pub enum Command {
     /// A command whose side effect is to define a shell function.
     Function(FunctionDefinition),
     /// A command that evaluates an extended test expression.
-    ExtendedTest(ExtendedTestExpr),
+    ExtendedTest(ExtendedTestExprCommand),
+}
+
+impl SourceLocation for Command {
+    fn location(&self) -> Option<TokenLocation> {
+        match self {
+            Self::Simple(s) => s.location(),
+            Self::Compound(c, r) => {
+                match (c.location(), r.as_ref().and_then(SourceLocation::location)) {
+                    (Some(s), Some(e)) => Some(TokenLocation::within(&s, &e)),
+                    (s, _) => s,
+                }
+            }
+            Self::Function(f) => f.location(),
+            Self::ExtendedTest(e) => e.location(),
+        }
+    }
 }
 
 impl Display for Command {
@@ -283,6 +383,23 @@ pub enum CompoundCommand {
     UntilClause(WhileOrUntilClauseCommand),
 }
 
+// TODO: complete the list
+impl SourceLocation for CompoundCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        match self {
+            Self::Arithmetic(a) => a.location(),
+            Self::ArithmeticForClause(a) => a.location(),
+            Self::BraceGroup(b) => b.location(),
+            Self::Subshell(s) => s.location(),
+            Self::ForClause(f) => f.location(),
+            Self::CaseClause(c) => c.location(),
+            Self::IfClause(i) => i.location(),
+            Self::WhileClause(w) => w.location(),
+            Self::UntilClause(u) => u.location(),
+        }
+    }
+}
+
 impl Display for CompoundCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -316,6 +433,14 @@ impl Display for CompoundCommand {
 pub struct ArithmeticCommand {
     /// The raw, unparsed and unexpanded arithmetic expression.
     pub expr: UnexpandedArithmeticExpr,
+    /// Location of the command
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for ArithmeticCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
 }
 
 impl Display for ArithmeticCommand {
@@ -328,12 +453,23 @@ impl Display for ArithmeticCommand {
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
-pub struct SubshellCommand(pub CompoundList);
+pub struct SubshellCommand {
+    /// Command list in the subshell
+    pub list: CompoundList,
+    /// Location of the subshell
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for SubshellCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
+}
 
 impl Display for SubshellCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "( ")?;
-        write!(f, "{}", self.0)?;
+        write!(f, "{}", self.list)?;
         write!(f, " )")
     }
 }
@@ -349,6 +485,14 @@ pub struct ForClauseCommand {
     pub values: Option<Vec<Word>>,
     /// The command to run for each iteration of the loop.
     pub body: DoGroupCommand,
+    /// Location of the for loop
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for ForClauseCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
 }
 
 impl Display for ForClauseCommand {
@@ -384,6 +528,14 @@ pub struct ArithmeticForClauseCommand {
     pub updater: Option<UnexpandedArithmeticExpr>,
     /// The command to run for each iteration of the loop.
     pub body: DoGroupCommand,
+    /// Location of the clause
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for ArithmeticForClauseCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
 }
 
 impl Display for ArithmeticForClauseCommand {
@@ -424,6 +576,15 @@ pub struct CaseClauseCommand {
     pub cases: Vec<CaseItem>,
 }
 
+impl SourceLocation for CaseClauseCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.value.location();
+        let end = self.cases.last().and_then(SourceLocation::location);
+
+        maybe_location(start.as_ref(), end.as_ref())
+    }
+}
+
 impl Display for CaseClauseCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "case {} in", self.value)?;
@@ -442,6 +603,20 @@ impl Display for CaseClauseCommand {
 #[cfg_attr(test, serde(rename = "List"))]
 pub struct CompoundList(pub Vec<CompoundListItem>);
 
+// TODO: doublecheck
+impl SourceLocation for CompoundList {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.0.first().and_then(SourceLocation::location);
+        let end = self.0.last().and_then(SourceLocation::location);
+
+        if let (Some(s), Some(e)) = (start, end) {
+            Some(TokenLocation::within(&s, &e))
+        } else {
+            None
+        }
+    }
+}
+
 impl Display for CompoundList {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for (i, item) in self.0.iter().enumerate() {
@@ -471,6 +646,12 @@ impl Display for CompoundList {
 #[cfg_attr(test, serde(rename = "Item"))]
 pub struct CompoundListItem(pub AndOrList, pub SeparatorOperator);
 
+impl SourceLocation for CompoundListItem {
+    fn location(&self) -> Option<TokenLocation> {
+        self.0.location()
+    }
+}
+
 impl Display for CompoundListItem {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.0)?;
@@ -491,6 +672,14 @@ pub struct IfClauseCommand {
     /// Optionally, `else` clauses that will be evaluated if the condition is false.
     #[cfg_attr(test, serde(skip_serializing_if = "Option::is_none"))]
     pub elses: Option<Vec<ElseClause>>,
+    /// Location of the if clause
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for IfClauseCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
 }
 
 impl Display for IfClauseCommand {
@@ -554,6 +743,14 @@ pub struct CaseItem {
     pub cmd: Option<CompoundList>,
     /// When the case branch is selected, the action to take after the command is executed.
     pub post_action: CaseItemPostAction,
+    /// Location of the item
+    pub loc: Option<TokenLocation>,
+}
+
+impl SourceLocation for CaseItem {
+    fn location(&self) -> Option<TokenLocation> {
+        self.loc.clone()
+    }
 }
 
 impl Display for CaseItem {
@@ -604,7 +801,13 @@ impl Display for CaseItemPostAction {
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
-pub struct WhileOrUntilClauseCommand(pub CompoundList, pub DoGroupCommand);
+pub struct WhileOrUntilClauseCommand(pub CompoundList, pub DoGroupCommand, pub TokenLocation);
+
+impl SourceLocation for WhileOrUntilClauseCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.2.clone())
+    }
+}
 
 impl Display for WhileOrUntilClauseCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -618,16 +821,29 @@ impl Display for WhileOrUntilClauseCommand {
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub struct FunctionDefinition {
     /// The name of the function.
-    pub fname: String,
+    pub fname: Word,
     /// The body of the function.
     pub body: FunctionBody,
     /// The source of the function definition.
     pub source: String,
 }
 
+impl SourceLocation for FunctionDefinition {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.fname.location();
+        let end = self.body.location();
+
+        if let (Some(s), Some(e)) = (start, end) {
+            Some(TokenLocation::within(&s, &e))
+        } else {
+            None
+        }
+    }
+}
+
 impl Display for FunctionDefinition {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        writeln!(f, "{} () ", self.fname)?;
+        writeln!(f, "{} () ", self.fname.value)?;
         write!(f, "{}", self.body)?;
         Ok(())
     }
@@ -639,6 +855,20 @@ impl Display for FunctionDefinition {
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub struct FunctionBody(pub CompoundCommand, pub Option<RedirectList>);
 
+impl SourceLocation for FunctionBody {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.0.location();
+
+        let end = self.1.as_ref().and_then(SourceLocation::location);
+
+        if let (Some(s), Some(e)) = (start, end) {
+            Some(TokenLocation::within(&s, &e))
+        } else {
+            None
+        }
+    }
+}
+
 impl Display for FunctionBody {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(f, "{}", self.0)?;
@@ -654,12 +884,27 @@ impl Display for FunctionBody {
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
-pub struct BraceGroupCommand(pub CompoundList);
+pub struct BraceGroupCommand {
+    /// List of commands
+    pub list: CompoundList,
+    /// Location of the group
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for BraceGroupCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
+}
 
 impl Display for BraceGroupCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(f, "{{ ")?;
-        write!(indenter::indented(f).with_str(DISPLAY_INDENT), "{}", self.0)?;
+        write!(
+            indenter::indented(f).with_str(DISPLAY_INDENT),
+            "{}",
+            self.list
+        )?;
         writeln!(f)?;
         write!(f, "}}")?;
 
@@ -671,12 +916,21 @@ impl Display for BraceGroupCommand {
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
-pub struct DoGroupCommand(pub CompoundList);
+pub struct DoGroupCommand {
+    /// List of commands
+    pub list: CompoundList,
+    /// Location of the group
+    pub loc: TokenLocation,
+}
 
 impl Display for DoGroupCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         writeln!(f, "do")?;
-        write!(indenter::indented(f).with_str(DISPLAY_INDENT), "{}", self.0)?;
+        write!(
+            indenter::indented(f).with_str(DISPLAY_INDENT),
+            "{}",
+            self.list
+        )?;
         writeln!(f)?;
         write!(f, "done")
     }
@@ -700,6 +954,22 @@ pub struct SimpleCommand {
     pub suffix: Option<CommandSuffix>,
 }
 
+impl SourceLocation for SimpleCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        let mid = &self
+            .word_or_name
+            .as_ref()
+            .and_then(SourceLocation::location);
+        let start = self.prefix.as_ref().and_then(SourceLocation::location);
+        let end = self.suffix.as_ref().and_then(SourceLocation::location);
+
+        maybe_location(
+            start.as_ref().or(mid.as_ref()),
+            end.as_ref().or(mid.as_ref()),
+        )
+    }
+}
+
 impl Display for SimpleCommand {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let mut wrote_something = false;
@@ -741,6 +1011,15 @@ impl Display for SimpleCommand {
 #[cfg_attr(test, serde(rename = "Prefix"))]
 pub struct CommandPrefix(pub Vec<CommandPrefixOrSuffixItem>);
 
+impl SourceLocation for CommandPrefix {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.0.first().and_then(SourceLocation::location);
+        let end = self.0.last().and_then(SourceLocation::location);
+
+        maybe_location(start.as_ref(), end.as_ref())
+    }
+}
+
 impl Display for CommandPrefix {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for (i, item) in self.0.iter().enumerate() {
@@ -761,6 +1040,15 @@ impl Display for CommandPrefix {
 #[cfg_attr(test, serde(rename = "Suffix"))]
 pub struct CommandSuffix(pub Vec<CommandPrefixOrSuffixItem>);
 
+impl SourceLocation for CommandSuffix {
+    fn location(&self) -> Option<TokenLocation> {
+        let start = self.0.first().and_then(SourceLocation::location);
+        let end = self.0.last().and_then(SourceLocation::location);
+
+        maybe_location(start.as_ref(), end.as_ref())
+    }
+}
+
 impl Display for CommandSuffix {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for (i, item) in self.0.iter().enumerate() {
@@ -810,6 +1098,16 @@ pub enum CommandPrefixOrSuffixItem {
     ProcessSubstitution(ProcessSubstitutionKind, SubshellCommand),
 }
 
+// TODO: complete
+impl SourceLocation for CommandPrefixOrSuffixItem {
+    fn location(&self) -> Option<TokenLocation> {
+        match self {
+            Self::Word(w) => w.location(),
+            _ => None,
+        }
+    }
+}
+
 impl Display for CommandPrefixOrSuffixItem {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -836,6 +1134,14 @@ pub struct Assignment {
     /// Whether or not to append to the preexisting value associated with the named variable.
     #[cfg_attr(test, serde(skip_serializing_if = "<&bool as std::ops::Not>::not"))]
     pub append: bool,
+    /// Location of the assignment
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for Assignment {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
 }
 
 impl Display for Assignment {
@@ -882,6 +1188,13 @@ pub enum AssignmentValue {
     Array(Vec<(Option<Word>, Word)>),
 }
 
+// TODO: complete
+impl SourceLocation for AssignmentValue {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 impl Display for AssignmentValue {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -909,6 +1222,13 @@ impl Display for AssignmentValue {
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub struct RedirectList(pub Vec<IoRedirect>);
 
+// TODO: complete
+impl SourceLocation for RedirectList {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 impl Display for RedirectList {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         for item in &self.0 {
@@ -918,21 +1238,31 @@ impl Display for RedirectList {
     }
 }
 
+/// A file descriptor number.
+pub type IoFd = i32;
+
 /// An I/O redirection.
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub enum IoRedirect {
     /// Redirection to a file.
-    File(Option<u32>, IoFileRedirectKind, IoFileRedirectTarget),
+    File(Option<IoFd>, IoFileRedirectKind, IoFileRedirectTarget),
     /// Redirection from a here-document.
-    HereDocument(Option<u32>, IoHereDocument),
+    HereDocument(Option<IoFd>, IoHereDocument),
     /// Redirection from a here-string.
-    HereString(Option<u32>, Word),
+    HereString(Option<IoFd>, Word),
     /// Redirection of both standard output and standard error (with optional append).
     OutputAndError(Word, bool),
 }
 
+// TODO: complete
+impl SourceLocation for IoRedirect {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 impl Display for IoRedirect {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -1029,7 +1359,7 @@ pub enum IoFileRedirectTarget {
     /// Path to a file.
     Filename(Word),
     /// File descriptor number.
-    Fd(u32),
+    Fd(IoFd),
     /// Process substitution: substitution with the results of executing the given
     /// command in a subshell.
     ProcessSubstitution(ProcessSubstitutionKind, SubshellCommand),
@@ -1069,6 +1399,13 @@ pub struct IoHereDocument {
     pub doc: Word,
 }
 
+// TODO: complete
+impl SourceLocation for IoHereDocument {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 /// A (non-extended) test expression.
 #[derive(Clone, Debug)]
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
@@ -1078,19 +1415,26 @@ pub enum TestExpr {
     /// A literal string.
     Literal(String),
     /// Logical AND operation on two nested expressions.
-    And(Box<TestExpr>, Box<TestExpr>),
+    And(Box<Self>, Box<Self>),
     /// Logical OR operation on two nested expressions.
-    Or(Box<TestExpr>, Box<TestExpr>),
+    Or(Box<Self>, Box<Self>),
     /// Logical NOT operation on a nested expression.
-    Not(Box<TestExpr>),
+    Not(Box<Self>),
     /// A parenthesized expression.
-    Parenthesized(Box<TestExpr>),
+    Parenthesized(Box<Self>),
     /// A unary test operation.
     UnaryTest(UnaryPredicate, String),
     /// A binary test operation.
     BinaryTest(BinaryPredicate, String, String),
 }
 
+// TODO: complete
+impl SourceLocation for TestExpr {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 impl Display for TestExpr {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -1112,13 +1456,13 @@ impl Display for TestExpr {
 #[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
 pub enum ExtendedTestExpr {
     /// Logical AND operation on two nested expressions.
-    And(Box<ExtendedTestExpr>, Box<ExtendedTestExpr>),
+    And(Box<Self>, Box<Self>),
     /// Logical OR operation on two nested expressions.
-    Or(Box<ExtendedTestExpr>, Box<ExtendedTestExpr>),
+    Or(Box<Self>, Box<Self>),
     /// Logical NOT operation on a nested expression.
-    Not(Box<ExtendedTestExpr>),
+    Not(Box<Self>),
     /// A parenthesized expression.
-    Parenthesized(Box<ExtendedTestExpr>),
+    Parenthesized(Box<Self>),
     /// A unary test operation.
     UnaryTest(UnaryPredicate, Word),
     /// A binary test operation.
@@ -1150,6 +1494,29 @@ impl Display for ExtendedTestExpr {
     }
 }
 
+/// An extended test expression command.
+#[derive(Clone, Debug)]
+#[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
+#[cfg_attr(test, derive(PartialEq, Eq, serde::Serialize))]
+pub struct ExtendedTestExprCommand {
+    /// The extended test expression
+    pub expr: ExtendedTestExpr,
+    /// Location of the expression
+    pub loc: TokenLocation,
+}
+
+impl SourceLocation for ExtendedTestExprCommand {
+    fn location(&self) -> Option<TokenLocation> {
+        Some(self.loc.clone())
+    }
+}
+
+impl Display for ExtendedTestExprCommand {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        self.expr.fmt(f)
+    }
+}
+
 /// A unary predicate usable in an extended test expression.
 #[derive(Clone, Debug)]
 #[cfg_attr(feature = "fuzz-testing", derive(arbitrary::Arbitrary))]
@@ -1313,6 +1680,14 @@ pub struct Word {
     /// Raw text of the word.
     #[cfg_attr(test, serde(rename = "v"))]
     pub value: String,
+    /// Location of the word
+    pub loc: Option<TokenLocation>,
+}
+
+impl SourceLocation for Word {
+    fn location(&self) -> Option<TokenLocation> {
+        self.loc.clone()
+    }
 }
 
 impl Display for Word {
@@ -1324,11 +1699,13 @@ impl Display for Word {
 impl From<&tokenizer::Token> for Word {
     fn from(t: &tokenizer::Token) -> Self {
         match t {
-            tokenizer::Token::Word(value, _) => Self {
+            tokenizer::Token::Word(value, loc) => Self {
                 value: value.clone(),
+                loc: Some(loc.clone()),
             },
-            tokenizer::Token::Operator(value, _) => Self {
+            tokenizer::Token::Operator(value, loc) => Self {
                 value: value.clone(),
+                loc: Some(loc.clone()),
             },
         }
     }
@@ -1336,7 +1713,10 @@ impl From<&tokenizer::Token> for Word {
 
 impl From<String> for Word {
     fn from(s: String) -> Self {
-        Self { value: s }
+        Self {
+            value: s,
+            loc: None,
+        }
     }
 }
 
@@ -1345,6 +1725,15 @@ impl Word {
     pub fn new(s: &str) -> Self {
         Self {
             value: s.to_owned(),
+            loc: None,
+        }
+    }
+
+    /// Constructs a new `Word` from a given string and location.
+    pub fn with_location(s: &str, loc: &TokenLocation) -> Self {
+        Self {
+            value: s.to_owned(),
+            loc: Some(loc.to_owned()),
         }
     }
 
@@ -1378,23 +1767,26 @@ pub enum ArithmeticExpr {
     /// A dereference of a variable or array element.
     Reference(ArithmeticTarget),
     /// A unary operation on an the result of a given nested expression.
-    UnaryOp(UnaryOperator, Box<ArithmeticExpr>),
+    UnaryOp(UnaryOperator, Box<Self>),
     /// A binary operation on two nested expressions.
-    BinaryOp(BinaryOperator, Box<ArithmeticExpr>, Box<ArithmeticExpr>),
+    BinaryOp(BinaryOperator, Box<Self>, Box<Self>),
     /// A ternary conditional expression.
-    Conditional(
-        Box<ArithmeticExpr>,
-        Box<ArithmeticExpr>,
-        Box<ArithmeticExpr>,
-    ),
+    Conditional(Box<Self>, Box<Self>, Box<Self>),
     /// An assignment operation.
-    Assignment(ArithmeticTarget, Box<ArithmeticExpr>),
+    Assignment(ArithmeticTarget, Box<Self>),
     /// A binary assignment operation.
-    BinaryAssignment(BinaryOperator, ArithmeticTarget, Box<ArithmeticExpr>),
+    BinaryAssignment(BinaryOperator, ArithmeticTarget, Box<Self>),
     /// A unary assignment operation.
     UnaryAssignment(UnaryAssignmentOperator, ArithmeticTarget),
 }
 
+// TODO: complete and add loc for literal
+impl SourceLocation for ArithmeticExpr {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 #[cfg(feature = "fuzz-testing")]
 impl<'a> arbitrary::Arbitrary<'a> for ArithmeticExpr {
     fn arbitrary(u: &mut arbitrary::Unstructured<'a>) -> arbitrary::Result<Self> {
@@ -1611,6 +2003,13 @@ pub enum ArithmeticTarget {
     ArrayElement(String, Box<ArithmeticExpr>),
 }
 
+// TODO: complete and add loc
+impl SourceLocation for ArithmeticTarget {
+    fn location(&self) -> Option<TokenLocation> {
+        None
+    }
+}
+
 impl Display for ArithmeticTarget {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
