@@ -22,7 +22,7 @@ pub use error::{
 #[cfg(feature = "diagnostics")]
 pub use error::miette::PrettyError;
 
-pub use parser::{Parser, ParserOptions, SourceInfo, parse_tokens};
+pub use parser::{Parser, ParserBuilder, ParserOptions, SourceInfo, parse_tokens};
 pub use tokenizer::{
     SourcePosition, Token, TokenLocation, TokenizerError, TokenizerOptions, tokenize_str,
     tokenize_str_with_options, uncached_tokenize_str, unquote_str,
