@@ -3,6 +3,9 @@
 #![allow(dead_code)]
 
 pub mod args;
+mod brushctl;
+pub mod entry;
+mod error_formatter;
 pub mod events;
-
 mod productinfo;
+mod shell_factory;
