@@ -10,7 +10,6 @@ use clap::Parser;
 use colored::Colorize;
 use descape::UnescapeExt;
 use serde::{Deserialize, Serialize};
-use std::fs;
 #[cfg(unix)]
 use std::os::unix::{fs::PermissionsExt, process::ExitStatusExt};
 use std::{
@@ -19,6 +18,7 @@ use std::{
     path::{Path, PathBuf},
     process::ExitStatus,
 };
+use std::{fs, io::Read};
 
 #[derive(Clone)]
 struct ShellConfig {
@@ -158,7 +158,7 @@ async fn cli_integration_tests(mut options: TestOptions) -> Result<()> {
 
     // Resolve path to the shell-under-test.
     if options.brush_path.is_empty() {
-        options.brush_path = assert_cmd::cargo::cargo_bin("brush")
+        options.brush_path = assert_cmd::cargo::cargo_bin!("brush")
             .to_string_lossy()
             .to_string();
     }
@@ -539,18 +539,13 @@ impl TestCaseSet {
     }
 }
 
-#[derive(Clone, Debug, Deserialize, Serialize)]
+#[derive(Clone, Debug, Default, Deserialize, Serialize)]
 enum ShellInvocation {
+    #[default]
     ExecShellBinary,
     ExecScript(String),
 }
 
-impl Default for ShellInvocation {
-    fn default() -> Self {
-        Self::ExecShellBinary
-    }
-}
-
 #[derive(Clone)]
 enum WhichShell {
     ShellUnderTest(PathBuf),
@@ -1178,8 +1173,7 @@ impl TestCase {
                 Err(anyhow::anyhow!("process was signaled"))
             }
             _ => Err(anyhow::anyhow!(
-                "unexpected status for process: {:?}",
-                wait_status
+                "unexpected status for process: {wait_status:?}"
             )),
         }
     }
@@ -1376,15 +1370,28 @@ fn diff_dirs(oracle_path: &Path, test_path: &Path) -> Result<DirComparison> {
         let test_file_path = test_path.join(filename);
 
         if file_type.is_file() {
-            let oracle_contents = fs::read_to_string(&oracle_file_path)?;
-            let test_contents = fs::read_to_string(&test_file_path)?;
+            let mut oracle_file = std::fs::OpenOptions::new()
+                .read(true)
+                .open(&oracle_file_path)?;
+            let mut oracle_bytes = vec![];
+            oracle_file.read_to_end(&mut oracle_bytes)?;
+
+            let mut test_file = std::fs::OpenOptions::new()
+                .read(true)
+                .open(&test_file_path)?;
+            let mut test_bytes = vec![];
+            test_file.read_to_end(&mut test_bytes)?;
+
+            if oracle_bytes != test_bytes {
+                // Convert using lossy conversion to avoid issues with invalid UTF-8.
+                let oracle_display_text = String::from_utf8_lossy(&oracle_bytes);
+                let test_display_text = String::from_utf8_lossy(&test_bytes);
 
-            if oracle_contents != test_contents {
                 entries.push(DirComparisonEntry::Different(
                     oracle_file_path,
-                    oracle_contents,
+                    oracle_display_text.to_string(),
                     test_file_path,
-                    test_contents,
+                    test_display_text.to_string(),
                 ));
             }
         } else if file_type.is_dir() {
